@@ -1,21 +1,217 @@
 
 var direction = 0;
 var component;
-var bullet;
+var bullet = null;
+
+var mysteryShip = null;
+var mysteryShipDeployedThisLevel = false;
+
 var aliendirection = 1;
 
 var aliens = new Array(10*5);
-var state = "stopped";
+var state = "STOPPED";
 
 var bombs = new Array(10); //Max of 10 bombs on screen at one time
 var score = 0;
 var dropBombChance = 2; //start chance of and alien dropping a bomb
+var alienStartSpeed = 600; //the speed the aliens move at teh start of the level
 
 var level = 0;
 var startlives = 3; //Edit this to cheat!
 var livesindication = new Array(startlives - 1);
 var lives;
 
+function cmdNotRunning() {
+    state = "NOTRUNNING";
+    level = 0;
+
+    //Show the settings button when the level starts
+    settingspanel.state = "HIDDEN";
+    hiscorepanel.state = "HIDDEN";
+    menupanel.state = "HIDDEN";
+
+    bunker1.demolish();
+    bunker2.demolish();
+    bunker3.demolish();
+    bunker4.demolish();
+
+    starttext.visible = true;
+
+    //Stop the animation timers
+    heartbeat.running = false;
+    alienanimation.running = false;
+
+    board.hideMessage();
+}
+
+function cmdNewGame() {
+    state = "NEWGAME";
+
+    Settings.loadSettings();
+
+    //Reset the score
+    score = 0;
+    scoretext.text = "Score: " + score;
+
+    //Reset the level
+    level = 0;
+
+    //Reset the alien speed and chance of dropping a bomb
+    alienStartSpeed = 600;
+    dropBombChance = 100;
+
+    lives = startlives;
+    destroyLivesIndication();
+    createLivesIndication();
+
+    cmdStartNewLevel();
+}
+
+function cmdRunning() {
+    state = "RUNNING";
+    board.hideMessage();
+
+    //Start the animation timers
+    heartbeat.running = true;
+    alienanimation.running = true;
+}
+
+function cmdLifeLost() {
+    state = "LIFELOST";
+    lives--;
+    if (livesindication[lives - 1]) {
+        livesindication[lives - 1].destroy();
+        livesindication[lives - 1] = null
+    }
+    board.displayMessage("Life Lost");
+
+    //Stop the animation timers
+    heartbeat.running = false;
+    alienanimation.running = false;
+
+}
+
+function cmdDead() {
+    state = "DEAD"
+
+    //Stop the animation timers
+    heartbeat.running = false;
+    alienanimation.running = false;
+
+    destroyAliens();
+    cleanupBombs();
+    bunker1.demolish();
+    bunker2.demolish();
+    bunker3.demolish();
+    bunker4.demolish();
+
+    board.displayMessage("GAME OVER");
+
+    //Show the seetings button when the game ends
+    settingspanel.state = "HIDDEN";
+    hiscorepanel.state = "HIDDEN";
+    menupanel.state = "HIDDEN"
+
+    Settings.addNewScore(level, score);
+    hiscorepanel.refreshScores();
+
+    deadtimer.start();
+}
+
+function cmdResume() {
+    cmdRunning();
+}
+
+function cmdLevelComplete() {
+    state = "LEVELCOMPLETE";
+
+    board.displayMessage("Level " + level + " complete");
+
+    //Remove the bunkers
+    bunker1.demolish();
+    bunker2.demolish();
+    bunker3.demolish();
+    bunker4.demolish();
+
+    starttext.visible = true;
+
+    heartbeat.running = false;
+    alienanimation.running = false;
+
+    cleanupBombs();
+
+    if (alienStartSpeed > 200) {
+        alienStartSpeed -= 50;
+    }
+
+    if (dropBombChance > 50) {
+        dropBombChance -= 10; //drop more bombs!
+    }
+}
+
+function cmdStartNewLevel() {
+    state = "LEVELSTARTING";
+
+    level++;
+    leveltext.text = "Level: " + level;
+
+    //Hide the panels when the level starts
+    settingspanel.state = "HIDDEN";
+    hiscorepanel.state = "HIDDEN";
+    menupanel.state = "HIDDEN";
+
+    bunker1.build();
+    bunker2.build();
+    bunker3.build();
+    bunker4.build();
+
+    mysteryShipDeployedThisLevel = false;
+
+    starttext.visible = false;
+    createAliens();
+
+    //Set the alien speed at the start of the level
+    alienanimation.interval = alienStartSpeed;
+
+    //Enable or disable the acceleromter
+    Settings.loadSettings();
+    if (Settings.UseAccelerometer == "TRUE" ) {
+        accelerometer.start();
+    } else {
+        accelerometer.stop();
+        direction = 0;
+    }
+
+    cmdRunning();
+}
+
+function cmdPause() {
+    state = "PAUSED";
+    board.displayMessage("Paused");
+
+    //Start the animation timers
+    heartbeat.running = false;
+    alienanimation.running = false;
+}
+
+function screenTap() {
+    if (settingspanel.state == "VISIBLE" || hiscorepanel.state == "VISIBLE" || menupanel.state == "VISIBLE") {
+        settingspanel.state = "HIDDEN";
+        hiscorepanel.state = "HIDDEN";
+        menupanel.state = "HIDDEN";
+        return;
+    }
+    if (state == "NOTRUNNING" && !deadtimer.running) {
+         cmdNewGame();
+    } else if (state == "LIFELOST" || state == "PAUSED") {
+        cmdResume();
+    } else if (state == "LEVELCOMPLETE") {
+        cmdStartNewLevel();
+    } else if (state == "RUNNING") {
+         fire();
+    }
+}
+
 function createLivesIndication() {
     var px = 0;
     var py = 0;
@@ -43,7 +239,7 @@ function createLivesIndication() {
 function destroyLivesIndication() {
     for (var i = 0; i < startlives-1; ++i) {
         if(livesindication[i]) {
-            livesinidication[i].destroy();
+            livesindication[i].destroy();
             livesindication[i] = null;
         }
     }
@@ -55,15 +251,9 @@ function shipHit() {
     }
 
     if (lives > 1) {
-        lives--;
-        if (livesindication[lives - 1]) {
-            livesindication[lives - 1].destroy();
-            livesindication[lives - 1] = null
-            lifelosttimer.start();
-        }
-
+       cmdLifeLost();
     } else { //game over
-        endGame(true);
+        cmdDead();
     }
 }
 
@@ -84,6 +274,70 @@ function createBullet() {
                 bullet.y = ship.y;
         }
     }
+}
+
+//Create all aliens and position them on the board in rows
+function createAliens() {
+
+    //Row 1
+    component = Qt.createComponent("alien1.qml");
+    var y = (Sizer.alien1height() + Sizer.alienYGap()) + Sizer.alienYGap() + 10;
+    for (var i=0; i<10; i++) {
+        aliens[i] = component.createObject(board);
+        aliens[i].x = ((aliens[i].width + Sizer.alienXGap()) * i) + Sizer.alienXGap();
+        aliens[i].y = y;
+    }
+
+    //Row 2
+    component = Qt.createComponent("alien2.qml");
+    y = ((Sizer.alien1height() + Sizer.alienYGap()) * 2) + Sizer.alienYGap() + 10;
+    for (var i=0; i<10; i++) {
+        aliens[i+10] = component.createObject(board);
+        aliens[i+10].x = ((aliens[i+10].width + Sizer.alienXGap()) * i) + Sizer.alienXGap();
+        aliens[i+10].y = y;
+    }
+
+    //Row 3
+    y = ((Sizer.alien1height() + Sizer.alienYGap()) * 3) + Sizer.alienYGap() + 10;
+    for (var i=0; i<10; i++) {
+        aliens[i+20] = component.createObject(board);
+        aliens[i+20].x = ((aliens[i+20].width + Sizer.alienXGap()) * i) + Sizer.alienXGap();
+        aliens[i+20].y = y;
+    }
+
+    //Row 4
+    component = Qt.createComponent("alien3.qml");
+    y = ((Sizer.alien1height() + Sizer.alienYGap()) * 4) + Sizer.alienYGap() + 10;
+    for (var i=0; i<10; i++) {
+        aliens[i+30] = component.createObject(board);
+        aliens[i+30].x = ((aliens[i+30].width + Sizer.alienXGap()) * i) + Sizer.alienXGap();
+        aliens[i+30].y = y;
+    }
+
+    //Row 5
+    y = ((Sizer.alien1height() + Sizer.alienYGap()) * 5) + Sizer.alienYGap() + 10;
+    for (var i=0; i<10; i++) {
+        aliens[i+40] = component.createObject(board);
+        aliens[i+40].x = ((aliens[i+40].width + Sizer.alienXGap()) * i) + Sizer.alienXGap();
+        aliens[i+40].y = y;
+    }
+}
+
+function rand(n)
+{
+  return (Math.floor(Math.random() * n));
+}
+
+function scheduleDirection(dir)
+{
+    direction = dir;
+}
+
+function fire()
+{
+    if (bullet == null) {
+        createBullet()
+    }
 
     //If the exisiting bullet is not off screen do nothing
     if (bullet.y > -10) {
@@ -99,203 +353,17 @@ function createBullet() {
         bullet.x = ship.x + ship.width/2 - bullet.width/2;
         bullet.y = ship.y
     }
-
-}
-
-//Create all aliens and position them on the board in rows
-function createAliens() {
-
-    //Row 1
-    component = Qt.createComponent("alien1.qml");
-    for (var i=0; i<10; i++) {
-        aliens[i] = component.createObject(board);
-        aliens[i].x = ((aliens[i].width + 10) * i) + 10;
-        aliens[i].y = 50;
-    }
-
-    //Row 2
-    component = Qt.createComponent("alien2.qml");
-    for (var i=0; i<10; i++) {
-        aliens[i+10] = component.createObject(board);
-        aliens[i+10].x = ((aliens[i+10].width + 10) * i) + 10;
-        aliens[i+10].y = 100;
-    }
-
-    //Row 3
-    for (var i=0; i<10; i++) {
-        aliens[i+20] = component.createObject(board);
-        aliens[i+20].x = ((aliens[i+20].width + 10) * i) + 10;
-        aliens[i+20].y = 150;
-    }
-
-    //Row 4
-    component = Qt.createComponent("alien3.qml");
-    for (var i=0; i<10; i++) {
-        aliens[i+30] = component.createObject(board);
-        aliens[i+30].x = ((aliens[i+30].width + 10) * i) + 10;
-        aliens[i+30].y = 200;
-    }
-
-    //Row 5
-    for (var i=0; i<10; i++) {
-        aliens[i+40] = component.createObject(board);
-        aliens[i+40].x = ((aliens[i+40].width + 10) * i) + 10;
-        aliens[i+40].y = 250;
-    }
-
-
-}
-
-function rand(n)
-{
-  return (Math.floor(Math.random() * n));
-}
-
-function scheduleDirection(dir)
-{
-    direction = dir;
-}
-
-function fire()
-{
-    if (state == "started") {
-        createBullet();
-    }
-}
-
-function startGame()
-{
-    if (state == "started")
-        return;
-
-    if (state == "stopped") {
-
-        Settings.loadSettings();
-
-        //Reset the score
-        score = 0;
-        scoretext.text = "Score: " + score;
-
-        //Reset the level
-        level = 1;
-        leveltext.text = "Level: " + level;
-
-        //Reset the alien speed and chance of dropping a bomb
-        alienanimation.interval = 600;
-        dropBombChance = 100;
-
-        lives = startlives;
-        destroyLivesIndication();
-        createLivesIndication();
-    }
-
-    //Hide the seetings button when the game starts
-    settingsbutton.visible = false;
-    settingspanel.state = "HIDDEN";
-    hiscorebutton.visible = false;
-    hiscorepanel.state = "HIDDEN";
-
-    Bunker1.create(96, 340, board);
-    Bunker2.create((96*2) + 80, 340, board);
-    Bunker3.create((96*3) + 160, 340, board);
-    Bunker4.create((96*4) + 220, 340, board);
-
-    starttext.visible = false;
-    createAliens();
-
-    //Start the animation timers
-    heartbeat.running = true;
-    alienanimation.running = true;
-
-    //Enable or disable the acceleromter
-    Settings.loadSettings();
-    if (Settings.UseAccelerometer == "TRUE" ) {
-        accelerometer.start();
-    } else {
-        accelerometer.stop();
-    }
-
-    gameover.visible = false;
-
-    state = "started";
-}
-
-function endGame(dead)
-{
-    state = "stopped";
-
-    //Stop the animation timers
-    heartbeat.running = false;
-    alienanimation.running = false;
-
-    destroyAliens();
-    cleanupBombs();
-    Bunker1.destroyBunker();
-    Bunker2.destroyBunker();
-    Bunker3.destroyBunker();
-    Bunker4.destroyBunker();
-
-    if (dead) {
-        gameover.text = "GAME OVER"
-    } else  {
-        gameover.text = "COMPLETE"
-    }
-
-    //Show the seetings button when the game ends
-    settingsbutton.visible = true;
-    settingspanel.state = "HIDDEN";
-    hiscorebutton.visible = true;
-    hiscorepanel.state = "HIDDEN";
-
-    Settings.addNewScore(level, score);
-    hiscorepanel.refreshScores();
-
-    deadtimer.start();
-    gameover.visible = true;
-}
-
-function nextLevel()
-{
-//    if (level < 5) {
-        state = "waitingforlevelstart"
-
-    //Remove the bunkers
-    Bunker1.destroyBunker();
-    Bunker2.destroyBunker();
-    Bunker3.destroyBunker();
-    Bunker4.destroyBunker();
-
-        starttext.visible = true;
-
-        level++;
-        leveltext.text = "Level: " + level;
-
-        heartbeat.running = false;
-        alienanimation.running = false;
-
-        cleanupBombs();
-
-        if (alienanimation.interval > 400) {
-            alienanimation.interval = 800 - (level * 50)
-        }
-
-        if (dropBombChance > 50) {
-            dropBombChance -= 10; //drop more bombs!
-        }
-//    } else {
-//        endGame(false);
-//    }
-
 }
 
 function mainEvent() {
-    move();
-    moveBombs();
-    moveBullet();
-    checkCollisions();
-
-    if (state == "started") {
+    if (state == "RUNNING") {
+        move();
+        moveBombs();
+        moveBullet();
+        mysteryShipHandling();
+        checkCollisions();
         checkComplete();
+
     }
 }
 
@@ -303,7 +371,7 @@ function moveBombs() {
     for (var i = 0; i < 10; ++i) {
         if (bombs[i]) {
             if (bombs[i].y  < board.height + 40) {
-               bombs[i].y += 10;
+               bombs[i].y += Sizer.bombSpeed();
             } else {
                bombs[i].destroy();
                bombs[i] = null;
@@ -315,33 +383,58 @@ function moveBombs() {
 function moveBullet() {
     if (bullet) {
         if (bullet.y > -20){
-           bullet.y -= 18;
+           bullet.y -= Sizer.laserSpeed();
         }
     }
 }
 
 function checkComplete() {
-    var count = 0;
-    for (var i=0; i<50; i++) {
-        if(aliens[i]) {
-            count++;
+    if (state == "RUNNING") {
+        var count = 0;
+        for (var i=0; i<50; i++) {
+            if(aliens[i]) {
+                count++;
+            }
         }
-    }
 
-    if(count == 0) {
-        //Complete!
-        nextLevel();
+        if(count == 0) {
+            //Complete!
+            cmdLevelComplete();
+        }
     }
 }
 
-function checkCollisions() {
+function checkCollisions()
+{
     checkBulletAlienCollision();
     checkBombShipCollision();
     checkBombBunkerCollision();
     checkBulletBunkerCollision();
+    checkBulletMysteryCollision();
 }
 
-function checkBulletAlienCollision() {
+function checkBulletMysteryCollision()
+{
+    if (bullet && mysteryShip) {
+        if (bullet.x > mysteryShip.x && bullet.x < mysteryShip.x + mysteryShip.width) {
+            if (bullet.y > mysteryShip.y && bullet.y < mysteryShip.y + mysteryShip.height) {
+                if (Settings.SFX == "TRUE"){
+                    destroyAlienSound.play();
+                }
+
+                score += mysteryShip.pointsAwarded;
+                scoretext.text = "Score: " + score;
+                bullet.y = -20; //Move the bullet offscreen
+
+                mysteryShip.destroy();
+                mysteryShip = null;
+            }
+        }
+    }
+}
+
+function checkBulletAlienCollision()
+{
     //Loop over each alien, checking if the bullet intersects
 
     for (var i=0; i<50; i++) {
@@ -352,14 +445,15 @@ function checkBulletAlienCollision() {
                         destroyAlienSound.play();
                     }
 
-                    aliens[i].destroy();
-                    aliens[i] = null;
-                    score += 100;
+                    score += aliens[i].pointsAwarded;
                     scoretext.text = "Score: " + score;
                     bullet.y = -20; //Move the bullet offscreen
 
-                    if (alienanimation.interval > 400) { //Get faster when a alien is killed
-                        alienanimation.interval -= 5;
+                    aliens[i].destroy();
+                    aliens[i] = null;
+
+                    if (alienanimation.interval > 100) { //Get faster when a alien is killed
+                        alienanimation.interval -= 10;
                     }
                     break;
                 }
@@ -393,32 +487,32 @@ function checkBombBunkerCollision() {
             var w = bombs[i].width;
             var h = bombs[i].height;
 
-            if (x >= Bunker1.x - w && x <= Bunker1.x + Bunker1.width){
-                if (Bunker1.checkBunkerHit(x, y, w, h)) {
+            if (x >= bunker1.x - w && x <= bunker1.x + bunker1.width){
+                if (bunker1.checkCollision(x, y, w, h)) {
                     bombs[i].destroy();
                     bombs[i] = null;
                     continue;
                 }
             }
 
-            if (x >= Bunker2.x - w && x <= Bunker2.x + Bunker2.width){
-                if (Bunker2.checkBunkerHit(x, y, w, h)) {
+            if (x >= bunker2.x - w && x <= bunker2.x + bunker2.width){
+                if (bunker2.checkCollision(x, y, w, h)) {
                     bombs[i].destroy();
                     bombs[i] = null;
                     continue;
                 }
             }
 
-            if (x >= Bunker3.x - w && x <= Bunker3.x + Bunker3.width){
-                if (Bunker3.checkBunkerHit(x, y, w, h)) {
+            if (x >= bunker3.x - w && x <= bunker3.x + bunker3.width){
+                if (bunker3.checkCollision(x, y, w, h)) {
                     bombs[i].destroy();
                     bombs[i] = null;
                     continue;
                 }
             }
 
-            if (x >= Bunker4.x - w && x <= Bunker4.x + Bunker4.width){
-                if (Bunker4.checkBunkerHit(x, y, w, h)) {
+            if (x >= bunker4.x - w && x <= bunker4.x + bunker4.width){
+                if (bunker4.checkCollision(x, y, w, h)) {
                     bombs[i].destroy();
                     bombs[i] = null;
                     continue;
@@ -435,16 +529,16 @@ function checkBulletBunkerCollision() {
         var w = bullet.width;
         var h = bullet.height;
 
-        if (Bunker1.checkBunkerHitRev(x, y, w, h)) {
+        if (bunker1.checkCollisionRev(x, y, w, h)) {
             bullet.y = -20;
         }
-        else if (Bunker2.checkBunkerHitRev(x, y, w, h)) {
+        else if (bunker2.checkCollisionRev(x, y, w, h)) {
             bullet.y = -20;
         }
-        else if (Bunker3.checkBunkerHitRev(x, y, w ,h)) {
+        else if (bunker3.checkCollisionRev(x, y, w ,h)) {
             bullet.y = -20;
         }
-        else if (Bunker4.checkBunkerHitRev(x, y, w, h)) {
+        else if (bunker4.checkCollisionRev(x, y, w, h)) {
             bullet.y = -20;
         }
     }
@@ -561,3 +655,59 @@ function move() {
     }
 }
 
+function mysteryShipHandling()
+{
+    //Check if there is already a mystery ship, and if so move it
+    //If there is no ship, decide whether or not to create one
+    if (mysteryShip == null) {
+        if (shouldDeployMysteryShip() == true) {
+            mysteryShipDeployedThisLevel = true;
+            component = Qt.createComponent("mystery.qml");
+
+
+            mysteryShip = component.createObject(board);
+            mysteryShip.x = board.width + 10;
+            mysteryShip.y = closebutton.height + 5;
+
+            mysteryShipSound.play();
+        }
+    } else {
+        if (mysteryShip.x > (0 - Sizer.mysteryWidth())) {
+            mysteryShip.x -= 10;
+        } else {
+            mysteryShipSound.stop();
+            mysteryShip.destroy();
+            mysteryShip = null;
+        }
+
+
+    }
+}
+
+function shouldDeployMysteryShip()
+{
+    //Check if already deplyed this level
+    if (mysteryShipDeployedThisLevel == true) {
+        return false;
+    }
+
+    //Check if there is space to deploy
+    var enoughSpace = true;
+
+    for (var i = 0; i < 10; ++i) {
+        if (aliens[i]) {
+            if (aliens[i].y < (Sizer.mysteryHeight() + closebutton.height + 5)) {
+                enoughSpace = false;
+            }
+        }
+    }
+
+    if (enoughSpace == false) {
+        return false;
+    } else {
+        if (rand(30) == 1){
+            return true;
+        }
+    }
+    return false;
+}
